using System.Web;
using System.Collections.Generic;
using System.Web.UI.WebControls;

using ExtendingTheGridView;

namespace ExtendedGridView
{
    /// <summary>
    /// Extended GridView
    /// Supports paging/sorting using a customized object data source.
    /// 
    /// Copyright (c) 2008 Kerry D. Wong (http://www.kerrywong.com)
    /// Permission is hereby granted, free of charge, to any person
    /// obtaining a copy of this software and associated documentation
    /// files (the "Software"), to deal in the Software without
    /// restriction, including without limitation the rights to use,
    /// copy, modify, merge, publish, distribute, sublicense, and/or sell
    /// copies of the Software, and to permit persons to whom the
    /// Software is furnished to do so, subject to the following
    /// conditions:
    /// 
    /// The above copyright notice and this permission notice shall be
    /// included in all copies or substantial portions of the Software.
    ///
    /// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    /// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
    /// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    /// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
    /// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
    /// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    /// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
    /// OTHER DEALINGS IN THE SOFTWARE.
    /// </summary>
    public class GridViewEx : GridView
    {
        public delegate void DataSourceChangedEventHandler(object sender, bool isEmpty);
        public event DataSourceChangedEventHandler DataSourceChanged;

        /// <summary>
        /// A readonly unique ID for the current view.
        /// </summary>
        public string CurrentViewID
        {
            get { return string.Concat("CurrentView_", UniqueID); }
        }

        /// <summary>
        /// Current page id when paging is used.
        /// To remember the page index user selected, a session variable is used.
        /// </summary>
        public int CurrentPageID
        {
            get
            {
                if (HttpContext.Current != null && HttpContext.Current.Session != null)
                {
                    if (HttpContext.Current.Session[string.Concat("CurrentPageID_", UniqueID)] == null)
                        return -1;
                    else
                        return int.Parse(HttpContext.Current.Session[string.Concat("CurrentPageID_", UniqueID)].ToString());
                }
                else
                    return -1;
            }
            set
            {
                if (HttpContext.Current != null && HttpContext.Current.Session != null)
                    HttpContext.Current.Session[string.Concat("CurrentPageID_", UniqueID)] = value;
            }
        }

        /// <summary>
        /// Overrides the data source property, so that we get a chance to test whether the source 
        /// being bound contains data or not.
        /// 
        /// This is used to communicate with the pager so that for an empty list no pager is shown.
        /// </summary>
        public override object DataSource
        {
            get
            {
                if (ViewState[CurrentViewID] != null)
                    return ViewState[CurrentViewID];
                else
                    return base.DataSource;
            }
            set
            {
                base.DataSource = value;
                ViewState[CurrentViewID] = value;

                if (value == null)
                {
                    if (DataSourceChanged != null)
                        DataSourceChanged(this, true);
                }
                else
                {
                    if (DataSourceChanged != null)
                        DataSourceChanged(this, false);
                }
            }
        }

        /// <summary>
        /// Set up event handlers.
        /// </summary>
        protected override void OnInit(System.EventArgs e)
        {
            AllowPaging = true;
            AllowSorting = true;
            this.AutoGenerateColumns = false;
            PageSize = 10;

            RowCancelingEdit += new GridViewCancelEditEventHandler(GridViewEx_RowCancelingEdit);
            RowDataBound += new GridViewRowEventHandler(GridViewEx_RowDataBound);
            RowEditing += new GridViewEditEventHandler(GridViewEx_RowEditing);

            base.OnInit(e);
        }

        protected override void OnPageIndexChanging(GridViewPageEventArgs e)
        {
            PageIndex = e.NewPageIndex;
            CurrentPageID = PageIndex;

            if (ViewState[CurrentViewID] != null)
                DataSource = ViewState[CurrentViewID];

            DataBind();
        }

        /// <summary>
        /// Handles the sort event.
        /// Since the data source is a generic list, we need to cast the list
        /// back to what ever type it is before we can sort. All the list types
        /// need to be handled here (see comments below)
        /// </summary>
        protected override void OnSorting(GridViewSortEventArgs e)
        {
            ViewState["SortExpression"] = e.SortExpression;

            if (ViewState[e.SortExpression] == null)
                ViewState[e.SortExpression] = e.SortDirection;
            else
            {
                ViewState[e.SortExpression] = (SortDirection)(1 - (int)ViewState[e.SortExpression]);
                e.SortDirection = (SortDirection)ViewState[e.SortExpression];
            }

            object o = null;

            //This is where more list types can be added. 
            if (ViewState[CurrentViewID] is List<ItemVal>)
            {
                List<ItemVal> l = ViewState[CurrentViewID] as List<ItemVal>;
                l.Sort(new GenericComparer<ItemVal>(e.SortDirection, e.SortExpression));
                o = l;
            }
            else
                base.OnSorting(e);

            DataSource = o;
            ViewState[CurrentViewID] = o;
            DataBind();
        }

        protected override void OnRowCreated(GridViewRowEventArgs e)
        {
            base.OnRowCreated(e);

            if (e.Row != null && e.Row.RowType == DataControlRowType.Header)
            {
                foreach (TableCell cell in e.Row.Cells)
                {
                    if (cell.HasControls())
                    {
                        LinkButton button = cell.Controls[0] as LinkButton;

                        if (button != null)
                        {
                            Image image = new Image();

                            string sortExpression = string.Empty;
                            SortDirection dir = SortDirection.Ascending;

                            if (ViewState["SortExpression"] != null)
                            {
                                sortExpression = ViewState["SortExpression"].ToString();
                                dir = (SortDirection)ViewState[sortExpression];
                            }

                            if (sortExpression != null && sortExpression == button.CommandArgument)
                            {
                                if (dir == SortDirection.Ascending)
                                    image.ImageUrl = "~/images/sort_asc.gif";
                                else
                                    image.ImageUrl = "~/images/sort_desc.gif";

                                cell.Controls.Add(image);
                            }
                        }
                    }
                }
            }
            else if (e.Row.RowType == DataControlRowType.DataRow) //Handles datarow highlighting on mouse move
            {
                e.Row.Attributes.Add("onmouseover", "this.originalstyle=this.style.backgroundColor;this.style.backgroundColor='lightgreen'");
                e.Row.Attributes.Add("onmouseout", "this.style.backgroundColor=this.originalstyle;");
            }
        }

        protected override void OnDataBinding(System.EventArgs e)
        {
            if (CurrentPageID != -1)
                PageIndex = CurrentPageID;

            base.OnDataBinding(e);
        }

        protected void GridViewEx_RowEditing(object sender, GridViewEditEventArgs e)
        {
            EditIndex = e.NewEditIndex;
            DataSource = ViewState[CurrentViewID];
            DataBind();
        }

        protected void GridViewEx_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            foreach (TableCell cell in e.Row.Cells)
            {
                foreach (System.Web.UI.Control c in cell.Controls)
                {
                    if (c is CheckBox)
                    {
                        CheckBox chkBox = c as CheckBox;

                        if (chkBox.Checked)
                        {
                            if (e.Row.RowState == DataControlRowState.Alternate)
                                e.Row.BackColor = System.Drawing.Color.LightSteelBlue;
                            else
                                e.Row.BackColor = System.Drawing.Color.AliceBlue;
                        }
                    }
                }
            }
        }

        protected void GridViewEx_RowCancelingEdit(object sender, GridViewCancelEditEventArgs e)
        {
            EditIndex = -1;
            DataSource = ViewState[CurrentViewID];
            DataBind();
        }

        public void PagerSelectedIndexChanged(object sender, int pageSize)
        {
            this.PageSize = pageSize;
            DataSource = ViewState[CurrentViewID];
            DataBind();
        }

        /// <summary>
        /// Calculate the real index since the actual index returned from the 
        /// GridView is the index in relation to the current page. We need the
        /// real index in order to index into the list.
        /// </summary>
        public int Index(int rowIndex)
        {
            return PageIndex * PageSize + rowIndex;
        }
    }
}
